@@ -39,7 +39,6 @@ func (er *ElasticsearchRequest) CreateOrUpdateRBAC() error {
 	if err != nil {
 		return kverrors.Wrap(err, "failed to create or update elasticsearch proxy clusterrole",
 			"cluster", dpl.Name,
-			"namespace", dpl.Namespace,
 		)
 	}
 
@@ -74,6 +73,43 @@ func (er *ElasticsearchRequest) CreateOrUpdateRBAC() error {
 		)
 	}
 
+	sccRole := rbac.NewRole(
+		"elasticsearch-restricted",
+		dpl.Namespace,
+		rbac.NewPolicyRules(
+			rbac.NewPolicyRule(
+				[]string{"security.openshift.io"},
+				[]string{"securitycontextconstraints"},
+				[]string{"elasticsearch-scc"},
+				[]string{"use"},
+				[]string{},
+			),
+		),
+	)
+
+	err = rbac.CreateOrUpdateRole(context.TODO(), er.client, sccRole)
+	if err != nil {
+		return kverrors.Wrap(err, "failed to create or update elasticsearch restricted role",
+			"cluster", dpl.Name,
+			"namespace", dpl.Namespace,
+		)
+	}
+
+	sccRoleBinding := rbac.NewRoleBinding(
+		"elasticsearch-restricted",
+		dpl.Namespace,
+		"elasticsearch-restricted",
+		subjects,
+	)
+
+	err = rbac.CreateOrUpdateRoleBinding(context.TODO(), er.client, sccRoleBinding)
+	if err != nil {
+		return kverrors.Wrap(err, "failed to create or update elasticsearch restricted rolebinding",
+			"role_binding_name", sccRoleBinding.Name,
+			"namespace", dpl.Namespace,
+		)
+	}
+
 	// metrics RBAC
 	metricsRole := rbac.NewRole(
 		"elasticsearch-metrics",
@@ -118,7 +154,7 @@ func (er *ElasticsearchRequest) CreateOrUpdateRBAC() error {
 		rbac.NewSubjects(subject),
 	)
 
-	err = rbac.CreateOrUpdateRoleBinding(context.TODO(), er.client, metricsRoleBinding)
+	err = rbac.CreateOrUpdateRoleBinding(context.TODO(),  er.client, metricsRoleBinding)
 	if err != nil {
 		return kverrors.Wrap(err, "failed to create or update elasticsearch metrics rolebinding",
 			"role_binding_name", metricsRoleBinding.Name,
